<?php declare(strict_types=1);

namespace App\Providers;

use App\Enums\Morph;
use App\Services\Search\CreateSearchInstanceCommand;
use App\Services\Search\Search;
use Illuminate\Cache\RateLimiting\Limit;
use Illuminate\Database\Eloquent\Relations\Relation;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\ServiceProvider;
use Illuminate\Validation\Rules\Password;
use Laravel\Sanctum\PersonalAccessToken;
use Laravel\Sanctum\Sanctum;

final class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     */
    public function register(): void
    {
        $this->app->bind(CreateSearchInstanceCommand::class, function () {
            return new CreateSearchInstanceCommand(Search::class);
        });
    }

    /**
     * Bootstrap any application services.
     */
    public function boot(): void
    {
        if (config('app.force_https') === true) {
            URL::forceScheme('https');
        }

        $this->passwordDefaults();

        Relation::enforceMorphMap(Morph::map());

        Route::pattern('username', '[a-z0-9]+(?:[._-][a-z0-9]+)*');
        Route::pattern('repository_name', '[a-z0-9]+(?:[._-][a-z0-9]+)*');
        Route::pattern('tag_name', '[a-zA-Z0-9]+(?:[._-][a-zA-Z0-9]+)*');
        Route::pattern('tag_id', '[0-9]+');

        $this->configureRateLimiting();
        Gate::define('AdminPanel', [\App\Policies\AdminPanel::class, 'view']);

        Sanctum::authenticateAccessTokensUsing(
            static function (PersonalAccessToken $accessToken, bool $is_valid) {
                if ($accessToken->tokenable?->is_active === true) {
                    return $is_valid;
                }

                return false;
            }
        );
    }

    /**
     * Configure the rate limiters for the application.
     */
    private function configureRateLimiting(): void
    {
        RateLimiter::for('login', function (Request $request) {
            return [
                Limit::perHour(config('rate_limiting.login_max_request', 50))->by($request->getClientIp()),
                Limit::perHour(config('rate_limiting.login_max_email_request', 10))->by($request->getClientIp() . '-' . $request->input('email')),
            ];
        });
    }

    private function passwordDefaults(): void
    {
        Password::defaults(function () {
            $rule = Password::min(8);

            if ($this->app->isProduction()) {
                $rule->letters()
                    ->mixedCase()
                    ->numbers()
                    ->symbols()
                    ->uncompromised();
            }

            return $rule;
        });
    }
}
